///////////////////////////////////////////////////////////////////////////////////
// float.h 
///////////////////////////////////////////////////////////////////////////////////
// BoostC floating point routines.
// This file containts the protypes of the maths functions
// 
// This work is a derivative of the SoftFloat IEC/IEEE Floating-point Arithmetic
// Package, Release 2b softfloat library written by John R. Hauser.
//
// License Agreement:
// This software may be freely used with the BoostC compiler and is subject to the
// SourceBoost software package license agreement.
// 
// Author: David Hobday
//
// Revision History:
//
// V1.0 - 25 Septmeber 2006
// Initial release. Code based on softfloat code. The code has been extensively 
// modified to make use on tiny targets viable, that is to reduce the RAM and 
// ROM usage.
//
// Copyright (C) 2006 David Hobday
// Copyright (C) 2006 Pavel Baranov
 
#ifndef _FLOAT_H_
#define _FLOAT_H_

typedef unsigned char uint8;
typedef signed long int32;
typedef unsigned char flag;

// Software IEC/IEEE floating-point underflow tininess-detection mode.
// Tininess detection mode can be determine or changed by examining or 
// modifying the global float_detect_tininess variable
extern uint8 float_detect_tininess;
enum 
{
    float_tininess_after_rounding  = 0,
    float_tininess_before_rounding = 1
};

// Software IEC/IEEE floating-point rounding mode.
// Rounding mode can be determine or changed by examining or 
// modifying the global float_rounding_mode variable
extern uint8 float_rounding_mode;
enum 
{
    float_round_nearest_even = 0,
    float_round_down         = 1,
    float_round_up           = 2,
    float_round_to_zero      = 3
};

// Software IEC/IEEE floating-point exception flags.
// Exception status can be tested or cleared by examining or 
// clearing the global float_exception_flags variable
extern uint8 float_exception_flags;
enum 
{
    float_flag_invalid   =  1,
    float_flag_divbyzero =  4,
    float_flag_overflow  =  8,
    float_flag_underflow = 16,
    float_flag_inexact   = 32
};

// Routine to raise any or all of the software IEC/IEEE floating-point
// exception flags.
#define float_raise( flags ) \
{\
    float_exception_flags |= flags;\
}

// Software IEC/IEEE integer-to-floating-point conversion routines.
float float32_from_int32( int32 );

//Software IEC/IEEE single-precision conversion routines.
int32 float32_to_int32( float );
int32 float32_to_int32_round_to_zero( float );

// Software IEC/IEEE single-precision operations.
float float32_round_to_int( float );
float float32_addsub( float, float, flag subtract ); // use by add and subtract
float float32_mul( float, float );
float float32_div( float, float );
float float32_rem( float, float );
float float32_sqrt( float );

#define float32_sub( a, b ) float32_addsub( a, b, 1 )
#define float32_add( a, b ) float32_addsub( a, b, 0 )

// will raise a float exception only if the argument values are Nan or signaling Nan
flag float32_le( float, float ); // The invalid exception is raised if either operand is a NaN
flag float32_lt( float, float ); // The invalid exception is raised if either operand is a NaN
flag float32_eq_signaling( float, float ); // The invalid exception is raised if either operand is a NaN
#define float32_gt( a, b ) float32_lt( b, a )
#define float32_ge( a, b ) float32_le( b, a )

// will raise a float exception only if the argument numbers are signalling Nan,
// with a Nan its quiet about it.
flag float32_le_quiet( float, float ); // The invalid exception is raised if either operand is a signalling NaN
flag float32_lt_quiet( float, float ); // The invalid exception is raised if either operand is a signalling NaN
flag float32_eq( float, float ); // The invalid exception is raised if either operand is a signalling NaN
#define float32_gt_quiet( a, b ) float32_lt_quiet( b, a )
#define float32_ge_quiet( a, b ) float32_le_quiet( b, a )

flag float32_is_signaling_nan( float );

#endif
